/* Path resolver for functions displaying paths (like perror_reply).
 * Makes an assumption, that if a `string' contains `guestpath', it
 * is a pathname to process.
 */

#ifdef NO_CHROOT

#include <stdio.h>
#include <param.h>
#include <string.h>

extern int guest, chrooted;
extern char guestpath[MAXPATHLEN+1];
char outpath[MAXPATHLEN+1];

char * fakepath(char *name)
{
	char *cp;

	if ((guest || chrooted) && (strstr(name, guestpath) != NULL))
	{
		cp = (char *)strchr(guestpath, '\0');
		name = name + (cp - guestpath);
		if (*name == '\0') name = "/";
	}
	return name;
}

/* Draco: path/filename resolver. Checks if it is allowed to access
 * the file user specified. If the path points to outside of the guest
 * account, it will be overwritten with a correct value.
 * BUG: filenames like `./foo.bar' won't be resolved properly.
 */

char * nresolve(char *name)
{
	char *cp;
	char opath[MAXPATHLEN]; 	/* Current path storage */
	char fname[MAXPATHLEN]; 	/* Filename part */
	extern char *getwd();

	if (guest || chrooted)		/* Only guests are restricted */
	{
		/* If the name contains the exact filename only
		 * we have nothing to do.
		 */
		if ((cp = strrchr(name,'/')) == NULL)
			return name;

	/* Preprocess the unresolved path */
		/* Hack for null pathnames: if the filename follows
		 * just the initial '/', we must not copy the slash.
		 */
		strcpy(fname, cp + (cp == name));
		*cp = '\0';			/* Cut the fname off */

		/* Save the current path before chdir() */
		getwd(opath);

		/* Paths starting with '/' must be redirected */
		if (name[0] == '/')
		{
			strcpy(outpath, guestpath);
			strcat(outpath, name);
		}
		else
			strcpy(outpath, name);

	/* Resolve the patch and process */
		/* Try to change directory to specified path.
		 * Errors will be processed by the parent function.
		 */
		if (chdir(outpath) < 0)
			strcat(outpath, fname);
		else
		{
			getwd(name);
			/* If the processed path is OK, overwrite
			 * the original path with the new one.
			 */
			if (strstr(name, guestpath) != NULL)
				strcpy(outpath, name);
			else
			{
			/* Add the guestpath at the begin otherwise
			 * and pass the whole thing to the caller.
			 */
				strcpy(outpath, guestpath);
				strcat(outpath, name);
			}
			/* Finally hook up the fname to the resolved path
			 */
			strcat(outpath, fname);
		}
		chdir(opath);
		return outpath;
	}
	return name;
}
#endif
